@@ -7,6 +7,7 @@ import math
 from scipy.signal import find_peaks
 import inspect
 import vxi11
+import time
 #================================================================
 # Valon_5015('Pete' or 'Ringo')
 #================================================================
@@ -480,10 +481,10 @@ class MOGLabs:
 #===========================================================================
 # Quantum_Composer (1.0 tested)
 #===========================================================================
-def sleep_method(method, *args, **kws):
+def sleep_method(method, *arg):
     t_sleep = 50e-3
-    def sleeping_method(self, *args, **kws):
-        method(self,*args, **kws)
+    def sleeping_method(self, *arg):
+        method(self,*arg)
         time.sleep(t_sleep)
         
     sleeping_method.__name__ = method.__name__
@@ -497,7 +498,7 @@ class Quantum_Composers:
     def __init__(self,address,*arg):
         if arg:
             self.address = address
-            self.baud_rate = arg
+            self.baud_rate = arg[0]
             self.rm = pyvisa.ResourceManager()
             self.dev = self.rm.open_resource(address, # same as 'ASRL5::INSTR'
                                   baud_rate = self.baud_rate, # must identify
@@ -718,12 +719,12 @@ class Quantum_Composers:
             self.__exp(__T0,8,'NORM',__tExp,0.000050,__tResp,1000,30000)
             self.on(1,2,5,6,8)
     @sleep_method
-    def burst(ch, n_pulses):
-        self.lev(self,ch)
-        self.pol(self,ch,'NORM')
-        self.norm(self,ch)
-        self.on(self,ch)
-        self.trigOff(self)
+    def burst(self, ch, n_pulses):
+        self.lev(ch)
+        self.pol(ch,'NORM')
+        self.norm(ch)
+        self.on(ch)
+        self.trigOff()
         self.dev.write(':PULSe{}:CMOD BURS'.format(ch))
         time.sleep(self.t_sleep)
         self.dev.write(':PULSe{}:BCOunter {}'.format(ch,n_pulses))
